@@ -5,17 +5,21 @@
 		- unpaper
 		- hocr2pdf (from ExactImage)
 	
-	(C) 2010-2016 Tobias Elze, modified for tesseract Heinrich Schwietering 2012
+	(C) 2010-2018 Tobias Elze, modified for tesseract Heinrich Schwietering 2012
 	patch for -rgb option contributed by James Cort
+	patch for a parallel computing problem with Tesseract 4, Dominique Meeus, 2018
 *)
 
 include Pdfsandwich_version;;		(*provides string variable pdfsandwich_version*)
 
+(*generate global temporary directory:*)
+let global_temp_dir = Filename.temp_file "pdfsandwich_tmp" "";;
+
 (*default binary names:*)
 let unpaper = ref "unpaper";;
 let identify = ref "identify";;
 let convert = ref "convert";;
-let tesseract = ref "tesseract";;
+let tesseract = ref "tesseract";;	(* this will be preceded by the OMP_THREAD_LIMIT (see options) *)
 let pdfinfo = ref "pdfinfo";;
 let pdfunite = ref "pdfunite";;
 let hocr2pdf = ref "hocr2pdf";;
@@ -30,6 +34,8 @@ let pr s =
 	if not !quiet then print_endline s
 ;;
 
+let make_temp_file = Filename.temp_file ~temp_dir:global_temp_dir "pdfsandwich";;
+
 (*execute command cmd and print it's invocation line (if verbose is set):*)
 let run ?(crash=true) cmd =
 	if !verbose then pr cmd;
@@ -127,12 +133,12 @@ let process_ocr
 		pr ("\nParallel processing with " ^ (string_of_int nthreads) ^ " threads started.\nProcessing page order may differ from original page order.\n");
 	
 	let process_page (curr_page, pdfname) =
-		let tmppicfile = Filename.temp_file "pdfsandwich" (if rgb then ".ppm" else if gray then ".pgm" else ".pbm") in
-		let tmptessinpfile = Filename.temp_file "pdfsandwich" ".tif" in
-		let tmpocrfile = Filename.temp_file "pdfsandwich" "" in
-		let tmpcolfigfile = Filename.temp_file "pdfsandwich" "_col.png" in
-		let tmprescaled_infile = Filename.temp_file "pdfsandwich" "_rescaled.pdf" in
-		let tmpunpaperfile = Filename.temp_file "pdfsandwich" ("_unpaper" ^ (if rgb then ".ppm" else if gray then ".pgm" else ".pbm")) in
+		let tmppicfile = make_temp_file (if rgb then ".ppm" else if gray then ".pgm" else ".pbm") in
+		let tmptessinpfile = make_temp_file ".tif" in
+		let tmpocrfile = make_temp_file "" in
+		let tmpcolfigfile = make_temp_file "_col.png" in
+		let tmprescaled_infile = make_temp_file "_rescaled.pdf" in
+		let tmpunpaperfile = make_temp_file ("_unpaper" ^ (if rgb then ".ppm" else if gray then ".pgm" else ".pbm")) in
 		if not !quiet then
 			Printf.printf "Processing page %i.\n" curr_page;
 		flush_all ();
@@ -196,7 +202,8 @@ let process_ocr
 			else 
 				if gray then " -colorspace gray -depth 8 -background white -flatten -alpha Off -density " else " -type Bilevel -density "
 		in
-		run (!convert ^ " -units PixelsPerInch " ^ convertopts ^ convoptstmp ^ (Printf.sprintf "%ix%i " resolution resolution)  ^ " \"" ^ convert_infile ^ "\" " ^  tmppicfile);
+		(*run (!convert ^ " -units PixelsPerInch " ^ convertopts ^ convoptstmp ^ (Printf.sprintf "%ix%i " resolution resolution)  ^ " \"" ^ convert_infile ^ "\" " ^  tmppicfile);*)
+		run (!convert ^ " -units PixelsPerInch "  ^ convoptstmp ^ (Printf.sprintf "%ix%i " resolution resolution) ^ convertopts ^ " \"" ^ convert_infile ^ "\" " ^  tmppicfile);
 		let tessout = if not !verbose then ">/dev/null 2>&1" else "" in
 		let preproc_output = 
 			if preprocess then
@@ -223,7 +230,7 @@ let process_ocr
 		else
 		(
 			if not !quiet then
-				print_endline "Tesseract was unable to produce a pdf ouput file. Possibly, version of tesseract is prior to 3.03 and cannot output pdf yet. Using hocr2pdf instead.";
+				print_endline "Tesseract was unable to produce a pdf output file. Possibly, version of tesseract is prior to 3.03 and cannot output pdf yet. Using hocr2pdf instead.";
 			run (!tesseract ^ " " ^ tessinputfile ^ tessout ^ " " ^ tmpocrfile ^ " " ^ tessopts ^ " -l " ^ language ^ " hocr ");
 			let hocrinputfile = if Sys.file_exists (tmpocrfile ^ ".html") then tmpocrfile ^ ".html" else tmpocrfile ^ ".hocr" in
 			run (!hocr2pdf ^ hocropts ^ " -r " ^ hocr_resolution ^ " -i " ^ tessinputfile ^ " -o " ^ pdfname ^ "<" ^ hocrinputfile);
@@ -245,7 +252,7 @@ let process_ocr
 	let process_pagelist = Array.iter process_page in
 	
 	let tmppdf_arr = 
-		Array.init pages_to_process (fun i -> (i+first_page, Filename.temp_file "pdfsandwich" ".pdf"))
+		Array.init pages_to_process (fun i -> (i+first_page, make_temp_file ".pdf"))
 	in
 	let intdiv = pages_to_process / nthreads
 	and remainder = pages_to_process mod nthreads in
@@ -273,7 +280,7 @@ let process_ocr
 	let tmpoutfile = 
 		if List.length pdffilenamelist > 1 then
 		(
-			let tout = Filename.temp_file "pdfsandwich_output" ".pdf" in
+			let tout = Filename.temp_file ~temp_dir:global_temp_dir "pdfsandwich_output" ".pdf" in
 			run (!pdfunite ^ " " ^ pdfliststring ^ " " ^ tout);
 			if (not debug) then List.iter Sys.remove pdffilenamelist;
 			tout
@@ -309,6 +316,9 @@ let main () =
 	let pagesize = ref "original" in
 	let set_hocr_opts op () = hocropts := !hocropts ^ " " ^ op in
 	let set_unp_opts op () = unpaperopts := !unpaperopts ^ " " ^ op in
+	(* determins the number of threads tesseract may use for each page;
+	if this is set to more than 1 it currently clashes with tesseract 4:*)
+	let omp_thread_limit = ref 1 in
 	
 	let get_tesseract_language_list () = 
 		let tesseract_langs =
@@ -358,6 +368,7 @@ let main () =
 		("-nopreproc", Arg.Clear preprocess, "\t do not preprocess with unpaper");
 		("-nthreads", Arg.Set_int nthreads, "\t -nthreads number : number of parallel threads (default: guessed number of CPUs; if guessing fails: 1)");
 		("-o", Arg.Set_string outputfile, "\t\t -o filename : output file; default: inputfile_ocr.pdf (if extension is different\n\t\t  from .pdf, original extension is kept)");
+		("-omp_thread_limit", Arg.Set_int omp_thread_limit, "\t -omp_thread_limit number : number of threads tesseract may use for each page (default: 1)\n\t\tvalues greater than 1 may cause tesseract >=4 to hang up");
 		("-pagesize", Arg.Set_string pagesize, "\t -pagesize { original | NUMxNUM } : set page size of output pdf (requires ghostscript)\n\t\t  original: same as input file (default)\n\t\t  NUMxNUM: width x height in pixel (e.g. for A4: -pagesize 595x842)");
 		("-pdfinfo", Arg.Set_string pdfinfo, "\t -pdfinfo filename : name of pdfinfo binary (default: pdfinfo)");
 		("-pdfunite", Arg.Set_string pdfunite, "\t -pdfunite filename : name of pdfunite binary (default: pdfunite)");
@@ -373,7 +384,10 @@ let main () =
 		("-version", Arg.Unit (fun () -> Printf.printf "pdfsandwich version %s\n" pdfsandwich_version; exit 0), "\t print version and quit");
 	] in
 	
+	Sys.remove global_temp_dir;
 	Arg.parse speclist (fun s -> arg_filename := s) "USAGE: pdfsandwich [options] inputfile.pdf\n\nOptions:";
+	Unix.mkdir global_temp_dir 0o700;
+	
 	List.iter check_for_binary [!convert; !tesseract; !gs; !pdfunite];
 	if !enforcehocr2pdf then check_for_binary !hocr2pdf;
 	if !preprocess then check_for_binary !unpaper;
@@ -425,7 +439,7 @@ let main () =
 	pr ("Input file: \"" ^ !arg_filename ^ "\"");
 	pr ("Output file: \"" ^ !outputfile ^ "\"");
 	(*use different filename for the input file in case the original input file contains special characters:*)
-	let internal_input_filename = Filename.temp_file "pdfsandwich_inputfile" ".pdf" in
+	let internal_input_filename = Filename.temp_file ~temp_dir:global_temp_dir "pdfsandwich_inputfile" ".pdf" in
 	Sys.remove internal_input_filename;
 	(*link original input file to internal_input_filename:*)
 	let full_path_input_file = if Filename.is_relative !arg_filename then ((Sys.getcwd ()) ^ "/" ^ !arg_filename) else !arg_filename in
@@ -444,6 +458,8 @@ let main () =
 		pr ("More threads than pages. Using " ^ (string_of_int npages_to_process) ^ " threads instead.");
 		nthreads := npages_to_process;
 	);
+	(*precede tesseract call with a thread limit to prevent hang ups for tesseract >=4:*)
+	tesseract := ("OMP_THREAD_LIMIT=" ^ (string_of_int !omp_thread_limit) ^ " " ^ !tesseract);
 	
 	process_ocr 
 		internal_input_filename 
@@ -462,8 +478,8 @@ let main () =
 		!maxpixels;
 	Unix.unlink internal_input_filename;
 	pr ("\n" ^ !outputfile ^ " generated.\n\nDone.");
+	if (not !debug) then Unix.rmdir global_temp_dir;
 ;;
 
 main ();;
 
-
